'''
Created on Mar 14, 2014

@author: Olivier Van Dam
'''

### IMPORTS
import logging
from org.jboss.dmr import ModelNode
from org.jboss.as.cli.scriptsupport import CLI
from xml.etree import ElementTree as ET

### VARIABLES - TODO - should be picked up from ???
host = "ServerHostForDomain"
port = 12345
login = "userLogin"
password = "userPassword"

jbossConfigXML = "/tmp/jbossConfig_OVD.xml"
jbossConfigLog = "/tmp/jbossConfig_OVD.log"
logLevel = logging.DEBUG

### FUNCTIONS

## general configuration

## build command for CLI
def runCLI(command):
	result = cli.cmd(command)
	if result.isSuccess():
		log = logging.info
	else:
		log = logging.error
	log(command)
	log("RESULT: %s" %(result.getResponse().get("outcome")))
	return result

def checkOperation(path, operation):
	# the list of operations we'll get is a java.util.List of ModelNode objects,
	# so to check if our operations is present, we need our operation wrapped as ModelNode
	opMN = ModelNode(operation)
	result = cli.cmd("%s:read-operation-names()" % (path))
	logging.debug("checkOperation - Result: %s" % (result.getResponse().get("result").asList()))
	operationExist = True
	if not result.isSuccess():
		logging.error("Could not check for operations at %s, does the path exist?" % (path))
		operationExist = False
	# check if operation is part of the list of available operations for this element
	elif not result.getResponse().get("result").asList().contains(opMN):
		logging.warning("Operation %s NOT available at %s" % (operation, path))
		operationExist = False
	return operationExist

def updateAttributes(path,attributes):
	operation = "write-attribute"
	if checkOperation(path, operation):
		for key, value in attributes.iteritems():
			runCLI('%s:%s(name="%s", value="%s"' % (path,operation,key,value))

def add(path,attributes):
	if cli.cmd("%s:read-resource()" %(path)).isSuccess():
		result = runCLI("%s:remove()" %(path))
		if not result.isSuccess():
			return result
	attributesList = []
	for key,value in attributes.iteritems():
		attributesList.append(key + '="' + value + '"')
	attributesStr = ','.join(attributesList)
	return runCLI("%s:add(%s)" % (path,attributesStr))

def remove(path,attributes):
	result = cli.cmd("%s:read-resource()" %(path))
	if result.isSuccess():
		result = runCLI("%s:remove()" %(path))
	return result

def update(path,attributes):
	operation = "update" 
	if len(attributes) > 0:
		if checkOperation(path,operation):
			attributesList = []
			for key,value in attributes.iteritems():
				attributesList.append(key + '="' + value + '"')
			attributesStr = ','.join(attributesList)
			# first check if element exists, otherwise we'll create it
			if cli.cmd("%s:read-resource()" %(path)).isSuccess():
				result = runCLI("%s:update(%s)" % (path,attributesStr))
			else:
				result = runCLI("%s:add(%s)" % (path,attributesStr))
			return result
		else:
			updateAttributes(path,attributes)
	else:
		logging.info("SKIPPING: %s - No attributes to update" % (path))

# TODO - allow attributes for enablement
def enable(path,attributes):
	if (cli.cmd("%s:read-resource()" %(path)).isSuccess() and 
		cli.cmd("%s:read-attribute(name=enabled)" %(path)).getResponse().get("result") == "false"):
		result = runCLI("%s:enable()" % (path))
	else:
		logging.debug("ALREADY ENABLED: %s" % (path))

def skip(path,attributes):
	logging.info("SKIPPING: " + path)

# links operation key to function
operationsDict = {
	"add":add,
	"remove":remove,
	"update":update,
	"enable":enable,
	"skip":skip
}
	
## manage recursive processing
def processing(element, path):
	logging.debug("Processing: %s/%s" % (path, element.tag))
	attributes = {}
	path = path + '/' + element.tag + '="' + element.attrib['name'] + '"'
	# we will iterate twice through the children, first to gather all "parameters" for elements at this level
	if 'operation' in element.attrib:
		operation = element.attrib['operation']
		for child in element.getchildren():
			# if the child element has no children of his own, it's a parameter to the comment at this level
			# only if no operation is defined at this level
			if 'operation' not in child.attrib and len(child.getchildren()) == 0:
				attributes[child.tag] = child.text
		# we have iterated and build up the attributes dictionary, now it is time to run the commands (if applicable)
		logging.debug("OPERATION: %s - %s" % (operation,attributes))
		operationsDict[operation](path, attributes)
	else:
		logging.debug("No operation defined for %s/%s, SKIPPING" % (path, element.tag))
	# second iteration, to call commands to be run at lower levels
	for child in element.getchildren():	
		if "name" in child.attrib:
			processing(child, path)

### MAIN Part
	
# setup the logging
logging.basicConfig(filename=jbossConfigLog,level=logLevel)

# let's connect to the server
cli = CLI.newInstance()
cli.connect(host, port, login, password)
config = ET.parse(jbossConfigXML)
root = config.getroot()

for child in root.getchildren():
	processing(child,'')
